#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include "lcd.h"

#define cpu_takt 160000000	//cpu_takt=16000000*10 da 10 perioden gewartet werden

volatile uint8_t flanke_anz=0;
volatile uint16_t zeit1=0;
volatile uint16_t zeit2=0;
volatile uint16_t ueberlauf=0;   // Anzahl Ueberlaeufe zwischen den Capture Interrupts

void wait (uint16_t zeit) //zeit: wartezeit in ms (0 bis 65536)
{
   uint16_t i;
   for(i=0;i<zeit;i++) _delay_ms(1); //da fr _delay_ms max Takt-abhngige Werte erlaubt
} 

int main (void)
{
  flanke_anz = 0;  // Anzahl der Flanken
  ueberlauf  = 0;  // Zhlt die Anzahl der berlufe des Timers von einem Capture zum nchsten


  char s[10];
  //uint32_t zeitdiff=0;			//differenz zwischen den 2 timestamp's
  double zeitdiff=0.0;
  double frequenz=0;

  DDRD &= ~( 1 << PD6 );		// PD6 als Eingang

  TCCR1B = ( 1 << CS10) | (1 << ICNC1 );       // capture steigende Flanke und prescaler 1
  TIMSK  = ( 1 << TICIE1 ) | ( 1 << TOIE1 );   // interrupt Enable, ovferflow Enable & Capture Enable


  lcd_init();
  lcd_string("Frequenzmessung");
  wait(500);
  sei();						//Interrupts aktivieren 


while(1)
{  
  //lcd_puts(itoa(flanke_anz,s,10));
  //lcd_puts("/n"); 
  if (flanke_anz>12) 
    { 
	  cli();	  
	  if (ueberlauf==0) 
	  {
  	    zeitdiff=zeit2-zeit1;			//wenn kein berlauf dann nur zeit zwischen timestamp's
	  }
	  else
	  {
	  //bsp fr 2 berlaufe:		
	  //      t1	1			2	t2
	  //    ...|-----|-----------|---|...
	  //      65535-zeit1		 zeit2
	  //		  65635*(ueberlauf-1)
	  //zeitdiff=(65535-zeit1)+65535*(ueberlauf-1)+zeit2;
	  //umgestellt:
	  zeitdiff=65635*ueberlauf+zeit2-zeit1;
	  //wenn berlauf, dann zeit von 1.timestamp zu 1.berlauf +zeit fr alle berlufe +zeit von letztem berlauf bis 2.timestamp
	  }
	  lcd_clear();
      lcd_string(ltoa(zeitdiff,s,10));	  
	  frequenz=cpu_takt/zeitdiff;
	  set_cursor(1,2);
	  lcd_string(dtostrf(frequenz,10,1,s));
	  ueberlauf=0;				//berlufe fr nchste Messung rcksetzen
	  flanke_anz=0;				//Rcksetzen fr nchste Messung
	  wait(500);
	  sei();
	}
}           
}


ISR(TIMER1_CAPT_vect)
{
   flanke_anz++;
   if (flanke_anz==3) 				// start erst bei 3. flanke
   		{
		  zeit1=ICR1;				//Time Stamp der 1.Flanke
		  ueberlauf=0;		  		//ab jetzt berlufe zhlen
		}				
   if (flanke_anz==13) 
   		{
		  zeit2=ICR1;				//Time Stamp der 10. Flanke
		}		
}


ISR( TIMER1_OVF_vect)
{
  ueberlauf++;
}


